# coding: utf-8

"""
    Xero Projects API

    This is the Xero Projects API  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ProjectPatch(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"status": "ProjectStatus"}

    attribute_map = {"status": "status"}

    def __init__(self, status=None):  # noqa: E501
        """ProjectPatch - a model defined in OpenAPI"""  # noqa: E501

        self._status = None
        self.discriminator = None

        self.status = status

    @property
    def status(self):
        """Gets the status of this ProjectPatch.  # noqa: E501


        :return: The status of this ProjectPatch.  # noqa: E501
        :rtype: ProjectStatus
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ProjectPatch.


        :param status: The status of this ProjectPatch.  # noqa: E501
        :type: ProjectStatus
        """
        if status is None:
            raise ValueError(
                "Invalid value for `status`, must not be `None`"
            )  # noqa: E501

        self._status = status



# -*- coding: utf-8 -*-
import functools
from types import MappingProxyType


def single_dispatch_str(func=None, key=None):
    """Single-dispatch generic function decorator.

    Transforms a function into a generic function, which can have different
    behaviours depending upon the string value of its first argument.
    The decorated function acts as the default implementation, and additional
    implementations can be registered using the register("key string") attribute
    of the generic function.

    you shall define @single_dispatch_str(key=get_key_function)
    get_key_function has to accept all parameters of the generic function

    """
    if func is None:
        return lambda f: single_dispatch_str(f, key)

    registry = {}

    # get function name for descriptive error messages
    func_name = getattr(func, "__name__", "single_dispatch_str function")

    def default_get_key(*args, **kwargs):
        """default strategy implementation, returns string value of its first argument"""
        if not args:
            raise TypeError(
                "{} requires at least 1 positional argument".format(func_name)
            )

        key = args[0]
        if not isinstance(key, str):
            raise TypeError(
                "{} requires first positional argument to be string type".format(
                    func_name
                )
            )
        return key

    get_key = key or default_get_key

    def dispatch(key):
        """generic_func.dispatch(key) -> <function implementation>

        Runs the dispatch algorithm to return the best available implementation
        for the given string *key* registered on *generic_func*.

        """
        try:
            impl = registry[key]
        except KeyError:
            impl = registry[""]  # default `func` implementation
        return impl

    def register(key, func=None):
        """generic_func.register(key, func) -> func

        Registers a new implementation for the given string *key*
        on a *generic_func*.

        """
        if not isinstance(key, str):
            raise TypeError(
                "{} requires string type key for register".format(func_name)
            )
        if not key:
            raise ValueError(
                "{} requires non empty string type key for register".format(func_name)
            )

        if func is None:
            return lambda f: register(key, f)
        registry[key] = func
        return func

    def wrapper(*args, **kw):
        """wrapper function replacing *generic_func*

        Triggers dispatch function to find registered implementation for
        first argument string value and executes these implementation

        """
        key = get_key(*args, **kw)
        if not isinstance(key, str):
            raise TypeError("{} requires key value to be string type".format(func_name))

        return dispatch(key)(*args, **kw)

    registry[""] = func
    wrapper.register = register
    wrapper.dispatch = dispatch
    wrapper.registry = MappingProxyType(registry)
    functools.update_wrapper(wrapper, func)
    return wrapper



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class ReimbursementType(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "name": "str",
        "account_code": "str",
        "reimbursement_type_id": "str",
        "updated_date_utc": "datetime[ms-format]",
        "current_record": "bool",
    }

    attribute_map = {
        "name": "Name",
        "account_code": "AccountCode",
        "reimbursement_type_id": "ReimbursementTypeID",
        "updated_date_utc": "UpdatedDateUTC",
        "current_record": "CurrentRecord",
    }

    def __init__(
        self,
        name=None,
        account_code=None,
        reimbursement_type_id=None,
        updated_date_utc=None,
        current_record=None,
    ):  # noqa: E501
        """ReimbursementType - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._account_code = None
        self._reimbursement_type_id = None
        self._updated_date_utc = None
        self._current_record = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if account_code is not None:
            self.account_code = account_code
        if reimbursement_type_id is not None:
            self.reimbursement_type_id = reimbursement_type_id
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if current_record is not None:
            self.current_record = current_record

    @property
    def name(self):
        """Gets the name of this ReimbursementType.  # noqa: E501

        Name of the earnings rate (max length = 100)  # noqa: E501

        :return: The name of this ReimbursementType.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ReimbursementType.

        Name of the earnings rate (max length = 100)  # noqa: E501

        :param name: The name of this ReimbursementType.  # noqa: E501
        :type: str
        """
        if name is not None and len(name) > 100:
            raise ValueError(
                "Invalid value for `name`, "
                "length must be less than or equal to `100`"
            )  # noqa: E501

        self._name = name

    @property
    def account_code(self):
        """Gets the account_code of this ReimbursementType.  # noqa: E501

        See Accounts  # noqa: E501

        :return: The account_code of this ReimbursementType.  # noqa: E501
        :rtype: str
        """
        return self._account_code

    @account_code.setter
    def account_code(self, account_code):
        """Sets the account_code of this ReimbursementType.

        See Accounts  # noqa: E501

        :param account_code: The account_code of this ReimbursementType.  # noqa: E501
        :type: str
        """

        self._account_code = account_code

    @property
    def reimbursement_type_id(self):
        """Gets the reimbursement_type_id of this ReimbursementType.  # noqa: E501

        Xero identifier  # noqa: E501

        :return: The reimbursement_type_id of this ReimbursementType.  # noqa: E501
        :rtype: str
        """
        return self._reimbursement_type_id

    @reimbursement_type_id.setter
    def reimbursement_type_id(self, reimbursement_type_id):
        """Sets the reimbursement_type_id of this ReimbursementType.

        Xero identifier  # noqa: E501

        :param reimbursement_type_id: The reimbursement_type_id of this ReimbursementType.  # noqa: E501
        :type: str
        """

        self._reimbursement_type_id = reimbursement_type_id

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this ReimbursementType.  # noqa: E501

        Last modified timestamp  # noqa: E501

        :return: The updated_date_utc of this ReimbursementType.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this ReimbursementType.

        Last modified timestamp  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this ReimbursementType.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def current_record(self):
        """Gets the current_record of this ReimbursementType.  # noqa: E501

        Is the current record  # noqa: E501

        :return: The current_record of this ReimbursementType.  # noqa: E501
        :rtype: bool
        """
        return self._current_record

    @current_record.setter
    def current_record(self, current_record):
        """Sets the current_record of this ReimbursementType.

        Is the current record  # noqa: E501

        :param current_record: The current_record of this ReimbursementType.  # noqa: E501
        :type: bool
        """

        self._current_record = current_record



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Item(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "code": "str",
        "inventory_asset_account_code": "str",
        "name": "str",
        "is_sold": "bool",
        "is_purchased": "bool",
        "description": "str",
        "purchase_description": "str",
        "purchase_details": "Purchase",
        "sales_details": "Purchase",
        "is_tracked_as_inventory": "bool",
        "total_cost_pool": "float",
        "quantity_on_hand": "float",
        "updated_date_utc": "datetime[ms-format]",
        "item_id": "str",
        "status_attribute_string": "str",
        "validation_errors": "list[ValidationError]",
    }

    attribute_map = {
        "code": "Code",
        "inventory_asset_account_code": "InventoryAssetAccountCode",
        "name": "Name",
        "is_sold": "IsSold",
        "is_purchased": "IsPurchased",
        "description": "Description",
        "purchase_description": "PurchaseDescription",
        "purchase_details": "PurchaseDetails",
        "sales_details": "SalesDetails",
        "is_tracked_as_inventory": "IsTrackedAsInventory",
        "total_cost_pool": "TotalCostPool",
        "quantity_on_hand": "QuantityOnHand",
        "updated_date_utc": "UpdatedDateUTC",
        "item_id": "ItemID",
        "status_attribute_string": "StatusAttributeString",
        "validation_errors": "ValidationErrors",
    }

    def __init__(
        self,
        code=None,
        inventory_asset_account_code=None,
        name=None,
        is_sold=None,
        is_purchased=None,
        description=None,
        purchase_description=None,
        purchase_details=None,
        sales_details=None,
        is_tracked_as_inventory=None,
        total_cost_pool=None,
        quantity_on_hand=None,
        updated_date_utc=None,
        item_id=None,
        status_attribute_string=None,
        validation_errors=None,
    ):  # noqa: E501
        """Item - a model defined in OpenAPI"""  # noqa: E501

        self._code = None
        self._inventory_asset_account_code = None
        self._name = None
        self._is_sold = None
        self._is_purchased = None
        self._description = None
        self._purchase_description = None
        self._purchase_details = None
        self._sales_details = None
        self._is_tracked_as_inventory = None
        self._total_cost_pool = None
        self._quantity_on_hand = None
        self._updated_date_utc = None
        self._item_id = None
        self._status_attribute_string = None
        self._validation_errors = None
        self.discriminator = None

        self.code = code
        if inventory_asset_account_code is not None:
            self.inventory_asset_account_code = inventory_asset_account_code
        if name is not None:
            self.name = name
        if is_sold is not None:
            self.is_sold = is_sold
        if is_purchased is not None:
            self.is_purchased = is_purchased
        if description is not None:
            self.description = description
        if purchase_description is not None:
            self.purchase_description = purchase_description
        if purchase_details is not None:
            self.purchase_details = purchase_details
        if sales_details is not None:
            self.sales_details = sales_details
        if is_tracked_as_inventory is not None:
            self.is_tracked_as_inventory = is_tracked_as_inventory
        if total_cost_pool is not None:
            self.total_cost_pool = total_cost_pool
        if quantity_on_hand is not None:
            self.quantity_on_hand = quantity_on_hand
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if item_id is not None:
            self.item_id = item_id
        if status_attribute_string is not None:
            self.status_attribute_string = status_attribute_string
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def code(self):
        """Gets the code of this Item.  # noqa: E501

        User defined item code (max length = 30)  # noqa: E501

        :return: The code of this Item.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this Item.

        User defined item code (max length = 30)  # noqa: E501

        :param code: The code of this Item.  # noqa: E501
        :type: str
        """
        if code is None:
            raise ValueError(
                "Invalid value for `code`, must not be `None`"
            )  # noqa: E501
        if code is not None and len(code) > 30:
            raise ValueError(
                "Invalid value for `code`, " "length must be less than or equal to `30`"
            )  # noqa: E501

        self._code = code

    @property
    def inventory_asset_account_code(self):
        """Gets the inventory_asset_account_code of this Item.  # noqa: E501

        The inventory asset account for the item. The account must be of type INVENTORY. The  COGSAccountCode in PurchaseDetails is also required to create a tracked item  # noqa: E501

        :return: The inventory_asset_account_code of this Item.  # noqa: E501
        :rtype: str
        """
        return self._inventory_asset_account_code

    @inventory_asset_account_code.setter
    def inventory_asset_account_code(self, inventory_asset_account_code):
        """Sets the inventory_asset_account_code of this Item.

        The inventory asset account for the item. The account must be of type INVENTORY. The  COGSAccountCode in PurchaseDetails is also required to create a tracked item  # noqa: E501

        :param inventory_asset_account_code: The inventory_asset_account_code of this Item.  # noqa: E501
        :type: str
        """

        self._inventory_asset_account_code = inventory_asset_account_code

    @property
    def name(self):
        """Gets the name of this Item.  # noqa: E501

        The name of the item (max length = 50)  # noqa: E501

        :return: The name of this Item.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Item.

        The name of the item (max length = 50)  # noqa: E501

        :param name: The name of this Item.  # noqa: E501
        :type: str
        """
        if name is not None and len(name) > 50:
            raise ValueError(
                "Invalid value for `name`, " "length must be less than or equal to `50`"
            )  # noqa: E501

        self._name = name

    @property
    def is_sold(self):
        """Gets the is_sold of this Item.  # noqa: E501

        Boolean value, defaults to true. When IsSold is true the item will be available on sales transactions in the Xero UI. If IsSold is updated to false then Description and SalesDetails values will be nulled.  # noqa: E501

        :return: The is_sold of this Item.  # noqa: E501
        :rtype: bool
        """
        return self._is_sold

    @is_sold.setter
    def is_sold(self, is_sold):
        """Sets the is_sold of this Item.

        Boolean value, defaults to true. When IsSold is true the item will be available on sales transactions in the Xero UI. If IsSold is updated to false then Description and SalesDetails values will be nulled.  # noqa: E501

        :param is_sold: The is_sold of this Item.  # noqa: E501
        :type: bool
        """

        self._is_sold = is_sold

    @property
    def is_purchased(self):
        """Gets the is_purchased of this Item.  # noqa: E501

        Boolean value, defaults to true. When IsPurchased is true the item is available for purchase transactions in the Xero UI. If IsPurchased is updated to false then PurchaseDescription and PurchaseDetails values will be nulled.  # noqa: E501

        :return: The is_purchased of this Item.  # noqa: E501
        :rtype: bool
        """
        return self._is_purchased

    @is_purchased.setter
    def is_purchased(self, is_purchased):
        """Sets the is_purchased of this Item.

        Boolean value, defaults to true. When IsPurchased is true the item is available for purchase transactions in the Xero UI. If IsPurchased is updated to false then PurchaseDescription and PurchaseDetails values will be nulled.  # noqa: E501

        :param is_purchased: The is_purchased of this Item.  # noqa: E501
        :type: bool
        """

        self._is_purchased = is_purchased

    @property
    def description(self):
        """Gets the description of this Item.  # noqa: E501

        The sales description of the item (max length = 4000)  # noqa: E501

        :return: The description of this Item.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Item.

        The sales description of the item (max length = 4000)  # noqa: E501

        :param description: The description of this Item.  # noqa: E501
        :type: str
        """
        if description is not None and len(description) > 4000:
            raise ValueError(
                "Invalid value for `description`, "
                "length must be less than or equal to `4000`"
            )  # noqa: E501

        self._description = description

    @property
    def purchase_description(self):
        """Gets the purchase_description of this Item.  # noqa: E501

        The purchase description of the item (max length = 4000)  # noqa: E501

        :return: The purchase_description of this Item.  # noqa: E501
        :rtype: str
        """
        return self._purchase_description

    @purchase_description.setter
    def purchase_description(self, purchase_description):
        """Sets the purchase_description of this Item.

        The purchase description of the item (max length = 4000)  # noqa: E501

        :param purchase_description: The purchase_description of this Item.  # noqa: E501
        :type: str
        """
        if purchase_description is not None and len(purchase_description) > 4000:
            raise ValueError(
                "Invalid value for `purchase_description`, "
                "length must be less than or equal to `4000`"
            )  # noqa: E501

        self._purchase_description = purchase_description

    @property
    def purchase_details(self):
        """Gets the purchase_details of this Item.  # noqa: E501


        :return: The purchase_details of this Item.  # noqa: E501
        :rtype: Purchase
        """
        return self._purchase_details

    @purchase_details.setter
    def purchase_details(self, purchase_details):
        """Sets the purchase_details of this Item.


        :param purchase_details: The purchase_details of this Item.  # noqa: E501
        :type: Purchase
        """

        self._purchase_details = purchase_details

    @property
    def sales_details(self):
        """Gets the sales_details of this Item.  # noqa: E501


        :return: The sales_details of this Item.  # noqa: E501
        :rtype: Purchase
        """
        return self._sales_details

    @sales_details.setter
    def sales_details(self, sales_details):
        """Sets the sales_details of this Item.


        :param sales_details: The sales_details of this Item.  # noqa: E501
        :type: Purchase
        """

        self._sales_details = sales_details

    @property
    def is_tracked_as_inventory(self):
        """Gets the is_tracked_as_inventory of this Item.  # noqa: E501

        True for items that are tracked as inventory. An item will be tracked as inventory if the InventoryAssetAccountCode and COGSAccountCode are set.  # noqa: E501

        :return: The is_tracked_as_inventory of this Item.  # noqa: E501
        :rtype: bool
        """
        return self._is_tracked_as_inventory

    @is_tracked_as_inventory.setter
    def is_tracked_as_inventory(self, is_tracked_as_inventory):
        """Sets the is_tracked_as_inventory of this Item.

        True for items that are tracked as inventory. An item will be tracked as inventory if the InventoryAssetAccountCode and COGSAccountCode are set.  # noqa: E501

        :param is_tracked_as_inventory: The is_tracked_as_inventory of this Item.  # noqa: E501
        :type: bool
        """

        self._is_tracked_as_inventory = is_tracked_as_inventory

    @property
    def total_cost_pool(self):
        """Gets the total_cost_pool of this Item.  # noqa: E501

        The value of the item on hand. Calculated using average cost accounting.  # noqa: E501

        :return: The total_cost_pool of this Item.  # noqa: E501
        :rtype: float
        """
        return self._total_cost_pool

    @total_cost_pool.setter
    def total_cost_pool(self, total_cost_pool):
        """Sets the total_cost_pool of this Item.

        The value of the item on hand. Calculated using average cost accounting.  # noqa: E501

        :param total_cost_pool: The total_cost_pool of this Item.  # noqa: E501
        :type: float
        """

        self._total_cost_pool = total_cost_pool

    @property
    def quantity_on_hand(self):
        """Gets the quantity_on_hand of this Item.  # noqa: E501

        The quantity of the item on hand  # noqa: E501

        :return: The quantity_on_hand of this Item.  # noqa: E501
        :rtype: float
        """
        return self._quantity_on_hand

    @quantity_on_hand.setter
    def quantity_on_hand(self, quantity_on_hand):
        """Sets the quantity_on_hand of this Item.

        The quantity of the item on hand  # noqa: E501

        :param quantity_on_hand: The quantity_on_hand of this Item.  # noqa: E501
        :type: float
        """

        self._quantity_on_hand = quantity_on_hand

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this Item.  # noqa: E501

        Last modified date in UTC format  # noqa: E501

        :return: The updated_date_utc of this Item.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this Item.

        Last modified date in UTC format  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this Item.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def item_id(self):
        """Gets the item_id of this Item.  # noqa: E501

        The Xero identifier for an Item  # noqa: E501

        :return: The item_id of this Item.  # noqa: E501
        :rtype: str
        """
        return self._item_id

    @item_id.setter
    def item_id(self, item_id):
        """Sets the item_id of this Item.

        The Xero identifier for an Item  # noqa: E501

        :param item_id: The item_id of this Item.  # noqa: E501
        :type: str
        """

        self._item_id = item_id

    @property
    def status_attribute_string(self):
        """Gets the status_attribute_string of this Item.  # noqa: E501

        Status of object  # noqa: E501

        :return: The status_attribute_string of this Item.  # noqa: E501
        :rtype: str
        """
        return self._status_attribute_string

    @status_attribute_string.setter
    def status_attribute_string(self, status_attribute_string):
        """Sets the status_attribute_string of this Item.

        Status of object  # noqa: E501

        :param status_attribute_string: The status_attribute_string of this Item.  # noqa: E501
        :type: str
        """

        self._status_attribute_string = status_attribute_string

    @property
    def validation_errors(self):
        """Gets the validation_errors of this Item.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this Item.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this Item.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this Item.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class BatchPayments(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"batch_payments": "list[BatchPayment]"}

    attribute_map = {"batch_payments": "BatchPayments"}

    def __init__(self, batch_payments=None):  # noqa: E501
        """BatchPayments - a model defined in OpenAPI"""  # noqa: E501

        self._batch_payments = None
        self.discriminator = None

        if batch_payments is not None:
            self.batch_payments = batch_payments

    @property
    def batch_payments(self):
        """Gets the batch_payments of this BatchPayments.  # noqa: E501


        :return: The batch_payments of this BatchPayments.  # noqa: E501
        :rtype: list[BatchPayment]
        """
        return self._batch_payments

    @batch_payments.setter
    def batch_payments(self, batch_payments):
        """Sets the batch_payments of this BatchPayments.


        :param batch_payments: The batch_payments of this BatchPayments.  # noqa: E501
        :type: list[BatchPayment]
        """

        self._batch_payments = batch_payments



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Overpayment(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "type": "str",
        "contact": "Contact",
        "date": "date[ms-format]",
        "status": "str",
        "line_amount_types": "LineAmountTypes",
        "line_items": "list[LineItem]",
        "sub_total": "float",
        "total_tax": "float",
        "total": "float",
        "updated_date_utc": "datetime[ms-format]",
        "currency_code": "CurrencyCode",
        "overpayment_id": "str",
        "currency_rate": "float",
        "remaining_credit": "float",
        "allocations": "list[Allocation]",
        "applied_amount": "float",
        "payments": "list[Payment]",
        "has_attachments": "bool",
        "attachments": "list[Attachment]",
    }

    attribute_map = {
        "type": "Type",
        "contact": "Contact",
        "date": "Date",
        "status": "Status",
        "line_amount_types": "LineAmountTypes",
        "line_items": "LineItems",
        "sub_total": "SubTotal",
        "total_tax": "TotalTax",
        "total": "Total",
        "updated_date_utc": "UpdatedDateUTC",
        "currency_code": "CurrencyCode",
        "overpayment_id": "OverpaymentID",
        "currency_rate": "CurrencyRate",
        "remaining_credit": "RemainingCredit",
        "allocations": "Allocations",
        "applied_amount": "AppliedAmount",
        "payments": "Payments",
        "has_attachments": "HasAttachments",
        "attachments": "Attachments",
    }

    def __init__(
        self,
        type=None,
        contact=None,
        date=None,
        status=None,
        line_amount_types=None,
        line_items=None,
        sub_total=None,
        total_tax=None,
        total=None,
        updated_date_utc=None,
        currency_code=None,
        overpayment_id=None,
        currency_rate=None,
        remaining_credit=None,
        allocations=None,
        applied_amount=None,
        payments=None,
        has_attachments=False,
        attachments=None,
    ):  # noqa: E501
        """Overpayment - a model defined in OpenAPI"""  # noqa: E501

        self._type = None
        self._contact = None
        self._date = None
        self._status = None
        self._line_amount_types = None
        self._line_items = None
        self._sub_total = None
        self._total_tax = None
        self._total = None
        self._updated_date_utc = None
        self._currency_code = None
        self._overpayment_id = None
        self._currency_rate = None
        self._remaining_credit = None
        self._allocations = None
        self._applied_amount = None
        self._payments = None
        self._has_attachments = None
        self._attachments = None
        self.discriminator = None

        if type is not None:
            self.type = type
        if contact is not None:
            self.contact = contact
        if date is not None:
            self.date = date
        if status is not None:
            self.status = status
        if line_amount_types is not None:
            self.line_amount_types = line_amount_types
        if line_items is not None:
            self.line_items = line_items
        if sub_total is not None:
            self.sub_total = sub_total
        if total_tax is not None:
            self.total_tax = total_tax
        if total is not None:
            self.total = total
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if currency_code is not None:
            self.currency_code = currency_code
        if overpayment_id is not None:
            self.overpayment_id = overpayment_id
        if currency_rate is not None:
            self.currency_rate = currency_rate
        if remaining_credit is not None:
            self.remaining_credit = remaining_credit
        if allocations is not None:
            self.allocations = allocations
        if applied_amount is not None:
            self.applied_amount = applied_amount
        if payments is not None:
            self.payments = payments
        if has_attachments is not None:
            self.has_attachments = has_attachments
        if attachments is not None:
            self.attachments = attachments

    @property
    def type(self):
        """Gets the type of this Overpayment.  # noqa: E501

        See Overpayment Types  # noqa: E501

        :return: The type of this Overpayment.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Overpayment.

        See Overpayment Types  # noqa: E501

        :param type: The type of this Overpayment.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "RECEIVE-OVERPAYMENT",
            "SPEND-OVERPAYMENT",
            "AROVERPAYMENT",
            "None",
        ]  # noqa: E501

        if type:
            if type not in allowed_values:
                raise ValueError(
                    "Invalid value for `type` ({0}), must be one of {1}".format(  # noqa: E501
                        type, allowed_values
                    )
                )

        self._type = type

    @property
    def contact(self):
        """Gets the contact of this Overpayment.  # noqa: E501


        :return: The contact of this Overpayment.  # noqa: E501
        :rtype: Contact
        """
        return self._contact

    @contact.setter
    def contact(self, contact):
        """Sets the contact of this Overpayment.


        :param contact: The contact of this Overpayment.  # noqa: E501
        :type: Contact
        """

        self._contact = contact

    @property
    def date(self):
        """Gets the date of this Overpayment.  # noqa: E501

        The date the overpayment is created YYYY-MM-DD  # noqa: E501

        :return: The date of this Overpayment.  # noqa: E501
        :rtype: date
        """
        return self._date

    @date.setter
    def date(self, date):
        """Sets the date of this Overpayment.

        The date the overpayment is created YYYY-MM-DD  # noqa: E501

        :param date: The date of this Overpayment.  # noqa: E501
        :type: date
        """

        self._date = date

    @property
    def status(self):
        """Gets the status of this Overpayment.  # noqa: E501

        See Overpayment Status Codes  # noqa: E501

        :return: The status of this Overpayment.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Overpayment.

        See Overpayment Status Codes  # noqa: E501

        :param status: The status of this Overpayment.  # noqa: E501
        :type: str
        """
        allowed_values = ["AUTHORISED", "PAID", "VOIDED", "None"]  # noqa: E501

        if status:
            if status not in allowed_values:
                raise ValueError(
                    "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                        status, allowed_values
                    )
                )

        self._status = status

    @property
    def line_amount_types(self):
        """Gets the line_amount_types of this Overpayment.  # noqa: E501


        :return: The line_amount_types of this Overpayment.  # noqa: E501
        :rtype: LineAmountTypes
        """
        return self._line_amount_types

    @line_amount_types.setter
    def line_amount_types(self, line_amount_types):
        """Sets the line_amount_types of this Overpayment.


        :param line_amount_types: The line_amount_types of this Overpayment.  # noqa: E501
        :type: LineAmountTypes
        """

        self._line_amount_types = line_amount_types

    @property
    def line_items(self):
        """Gets the line_items of this Overpayment.  # noqa: E501

        See Overpayment Line Items  # noqa: E501

        :return: The line_items of this Overpayment.  # noqa: E501
        :rtype: list[LineItem]
        """
        return self._line_items

    @line_items.setter
    def line_items(self, line_items):
        """Sets the line_items of this Overpayment.

        See Overpayment Line Items  # noqa: E501

        :param line_items: The line_items of this Overpayment.  # noqa: E501
        :type: list[LineItem]
        """

        self._line_items = line_items

    @property
    def sub_total(self):
        """Gets the sub_total of this Overpayment.  # noqa: E501

        The subtotal of the overpayment excluding taxes  # noqa: E501

        :return: The sub_total of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._sub_total

    @sub_total.setter
    def sub_total(self, sub_total):
        """Sets the sub_total of this Overpayment.

        The subtotal of the overpayment excluding taxes  # noqa: E501

        :param sub_total: The sub_total of this Overpayment.  # noqa: E501
        :type: float
        """

        self._sub_total = sub_total

    @property
    def total_tax(self):
        """Gets the total_tax of this Overpayment.  # noqa: E501

        The total tax on the overpayment  # noqa: E501

        :return: The total_tax of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._total_tax

    @total_tax.setter
    def total_tax(self, total_tax):
        """Sets the total_tax of this Overpayment.

        The total tax on the overpayment  # noqa: E501

        :param total_tax: The total_tax of this Overpayment.  # noqa: E501
        :type: float
        """

        self._total_tax = total_tax

    @property
    def total(self):
        """Gets the total of this Overpayment.  # noqa: E501

        The total of the overpayment (subtotal + total tax)  # noqa: E501

        :return: The total of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this Overpayment.

        The total of the overpayment (subtotal + total tax)  # noqa: E501

        :param total: The total of this Overpayment.  # noqa: E501
        :type: float
        """

        self._total = total

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this Overpayment.  # noqa: E501

        UTC timestamp of last update to the overpayment  # noqa: E501

        :return: The updated_date_utc of this Overpayment.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this Overpayment.

        UTC timestamp of last update to the overpayment  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this Overpayment.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def currency_code(self):
        """Gets the currency_code of this Overpayment.  # noqa: E501


        :return: The currency_code of this Overpayment.  # noqa: E501
        :rtype: CurrencyCode
        """
        return self._currency_code

    @currency_code.setter
    def currency_code(self, currency_code):
        """Sets the currency_code of this Overpayment.


        :param currency_code: The currency_code of this Overpayment.  # noqa: E501
        :type: CurrencyCode
        """

        self._currency_code = currency_code

    @property
    def overpayment_id(self):
        """Gets the overpayment_id of this Overpayment.  # noqa: E501

        Xero generated unique identifier  # noqa: E501

        :return: The overpayment_id of this Overpayment.  # noqa: E501
        :rtype: str
        """
        return self._overpayment_id

    @overpayment_id.setter
    def overpayment_id(self, overpayment_id):
        """Sets the overpayment_id of this Overpayment.

        Xero generated unique identifier  # noqa: E501

        :param overpayment_id: The overpayment_id of this Overpayment.  # noqa: E501
        :type: str
        """

        self._overpayment_id = overpayment_id

    @property
    def currency_rate(self):
        """Gets the currency_rate of this Overpayment.  # noqa: E501

        The currency rate for a multicurrency overpayment. If no rate is specified, the XE.com day rate is used  # noqa: E501

        :return: The currency_rate of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._currency_rate

    @currency_rate.setter
    def currency_rate(self, currency_rate):
        """Sets the currency_rate of this Overpayment.

        The currency rate for a multicurrency overpayment. If no rate is specified, the XE.com day rate is used  # noqa: E501

        :param currency_rate: The currency_rate of this Overpayment.  # noqa: E501
        :type: float
        """

        self._currency_rate = currency_rate

    @property
    def remaining_credit(self):
        """Gets the remaining_credit of this Overpayment.  # noqa: E501

        The remaining credit balance on the overpayment  # noqa: E501

        :return: The remaining_credit of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._remaining_credit

    @remaining_credit.setter
    def remaining_credit(self, remaining_credit):
        """Sets the remaining_credit of this Overpayment.

        The remaining credit balance on the overpayment  # noqa: E501

        :param remaining_credit: The remaining_credit of this Overpayment.  # noqa: E501
        :type: float
        """

        self._remaining_credit = remaining_credit

    @property
    def allocations(self):
        """Gets the allocations of this Overpayment.  # noqa: E501

        See Allocations  # noqa: E501

        :return: The allocations of this Overpayment.  # noqa: E501
        :rtype: list[Allocation]
        """
        return self._allocations

    @allocations.setter
    def allocations(self, allocations):
        """Sets the allocations of this Overpayment.

        See Allocations  # noqa: E501

        :param allocations: The allocations of this Overpayment.  # noqa: E501
        :type: list[Allocation]
        """

        self._allocations = allocations

    @property
    def applied_amount(self):
        """Gets the applied_amount of this Overpayment.  # noqa: E501

        The amount of applied to an invoice  # noqa: E501

        :return: The applied_amount of this Overpayment.  # noqa: E501
        :rtype: float
        """
        return self._applied_amount

    @applied_amount.setter
    def applied_amount(self, applied_amount):
        """Sets the applied_amount of this Overpayment.

        The amount of applied to an invoice  # noqa: E501

        :param applied_amount: The applied_amount of this Overpayment.  # noqa: E501
        :type: float
        """

        self._applied_amount = applied_amount

    @property
    def payments(self):
        """Gets the payments of this Overpayment.  # noqa: E501

        See Payments  # noqa: E501

        :return: The payments of this Overpayment.  # noqa: E501
        :rtype: list[Payment]
        """
        return self._payments

    @payments.setter
    def payments(self, payments):
        """Sets the payments of this Overpayment.

        See Payments  # noqa: E501

        :param payments: The payments of this Overpayment.  # noqa: E501
        :type: list[Payment]
        """

        self._payments = payments

    @property
    def has_attachments(self):
        """Gets the has_attachments of this Overpayment.  # noqa: E501

        boolean to indicate if a overpayment has an attachment  # noqa: E501

        :return: The has_attachments of this Overpayment.  # noqa: E501
        :rtype: bool
        """
        return self._has_attachments

    @has_attachments.setter
    def has_attachments(self, has_attachments):
        """Sets the has_attachments of this Overpayment.

        boolean to indicate if a overpayment has an attachment  # noqa: E501

        :param has_attachments: The has_attachments of this Overpayment.  # noqa: E501
        :type: bool
        """

        self._has_attachments = has_attachments

    @property
    def attachments(self):
        """Gets the attachments of this Overpayment.  # noqa: E501

        See Attachments  # noqa: E501

        :return: The attachments of this Overpayment.  # noqa: E501
        :rtype: list[Attachment]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments):
        """Sets the attachments of this Overpayment.

        See Attachments  # noqa: E501

        :param attachments: The attachments of this Overpayment.  # noqa: E501
        :type: list[Attachment]
        """

        self._attachments = attachments



# coding: utf-8

"""
    Xero Assets API

    The Assets API exposes fixed asset related functions of the Xero Accounting application and can be used for a variety of purposes such as creating assets, retrieving asset valuations etc.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class AssetType(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "asset_type_id": "str",
        "asset_type_name": "str",
        "fixed_asset_account_id": "str",
        "depreciation_expense_account_id": "str",
        "accumulated_depreciation_account_id": "str",
        "book_depreciation_setting": "BookDepreciationSetting",
        "locks": "int",
    }

    attribute_map = {
        "asset_type_id": "assetTypeId",
        "asset_type_name": "assetTypeName",
        "fixed_asset_account_id": "fixedAssetAccountId",
        "depreciation_expense_account_id": "depreciationExpenseAccountId",
        "accumulated_depreciation_account_id": "accumulatedDepreciationAccountId",
        "book_depreciation_setting": "bookDepreciationSetting",
        "locks": "locks",
    }

    def __init__(
        self,
        asset_type_id=None,
        asset_type_name=None,
        fixed_asset_account_id=None,
        depreciation_expense_account_id=None,
        accumulated_depreciation_account_id=None,
        book_depreciation_setting=None,
        locks=None,
    ):  # noqa: E501
        """AssetType - a model defined in OpenAPI"""  # noqa: E501

        self._asset_type_id = None
        self._asset_type_name = None
        self._fixed_asset_account_id = None
        self._depreciation_expense_account_id = None
        self._accumulated_depreciation_account_id = None
        self._book_depreciation_setting = None
        self._locks = None
        self.discriminator = None

        if asset_type_id is not None:
            self.asset_type_id = asset_type_id
        self.asset_type_name = asset_type_name
        if fixed_asset_account_id is not None:
            self.fixed_asset_account_id = fixed_asset_account_id
        if depreciation_expense_account_id is not None:
            self.depreciation_expense_account_id = depreciation_expense_account_id
        if accumulated_depreciation_account_id is not None:
            self.accumulated_depreciation_account_id = (
                accumulated_depreciation_account_id
            )
        self.book_depreciation_setting = book_depreciation_setting
        if locks is not None:
            self.locks = locks

    @property
    def asset_type_id(self):
        """Gets the asset_type_id of this AssetType.  # noqa: E501

        Xero generated unique identifier for asset types  # noqa: E501

        :return: The asset_type_id of this AssetType.  # noqa: E501
        :rtype: str
        """
        return self._asset_type_id

    @asset_type_id.setter
    def asset_type_id(self, asset_type_id):
        """Sets the asset_type_id of this AssetType.

        Xero generated unique identifier for asset types  # noqa: E501

        :param asset_type_id: The asset_type_id of this AssetType.  # noqa: E501
        :type: str
        """

        self._asset_type_id = asset_type_id

    @property
    def asset_type_name(self):
        """Gets the asset_type_name of this AssetType.  # noqa: E501

        The name of the asset type  # noqa: E501

        :return: The asset_type_name of this AssetType.  # noqa: E501
        :rtype: str
        """
        return self._asset_type_name

    @asset_type_name.setter
    def asset_type_name(self, asset_type_name):
        """Sets the asset_type_name of this AssetType.

        The name of the asset type  # noqa: E501

        :param asset_type_name: The asset_type_name of this AssetType.  # noqa: E501
        :type: str
        """
        if asset_type_name is None:
            raise ValueError(
                "Invalid value for `asset_type_name`, must not be `None`"
            )  # noqa: E501

        self._asset_type_name = asset_type_name

    @property
    def fixed_asset_account_id(self):
        """Gets the fixed_asset_account_id of this AssetType.  # noqa: E501

        The asset account for fixed assets of this type  # noqa: E501

        :return: The fixed_asset_account_id of this AssetType.  # noqa: E501
        :rtype: str
        """
        return self._fixed_asset_account_id

    @fixed_asset_account_id.setter
    def fixed_asset_account_id(self, fixed_asset_account_id):
        """Sets the fixed_asset_account_id of this AssetType.

        The asset account for fixed assets of this type  # noqa: E501

        :param fixed_asset_account_id: The fixed_asset_account_id of this AssetType.  # noqa: E501
        :type: str
        """

        self._fixed_asset_account_id = fixed_asset_account_id

    @property
    def depreciation_expense_account_id(self):
        """Gets the depreciation_expense_account_id of this AssetType.  # noqa: E501

        The expense account for the depreciation of fixed assets of this type  # noqa: E501

        :return: The depreciation_expense_account_id of this AssetType.  # noqa: E501
        :rtype: str
        """
        return self._depreciation_expense_account_id

    @depreciation_expense_account_id.setter
    def depreciation_expense_account_id(self, depreciation_expense_account_id):
        """Sets the depreciation_expense_account_id of this AssetType.

        The expense account for the depreciation of fixed assets of this type  # noqa: E501

        :param depreciation_expense_account_id: The depreciation_expense_account_id of this AssetType.  # noqa: E501
        :type: str
        """

        self._depreciation_expense_account_id = depreciation_expense_account_id

    @property
    def accumulated_depreciation_account_id(self):
        """Gets the accumulated_depreciation_account_id of this AssetType.  # noqa: E501

        The account for accumulated depreciation of fixed assets of this type  # noqa: E501

        :return: The accumulated_depreciation_account_id of this AssetType.  # noqa: E501
        :rtype: str
        """
        return self._accumulated_depreciation_account_id

    @accumulated_depreciation_account_id.setter
    def accumulated_depreciation_account_id(self, accumulated_depreciation_account_id):
        """Sets the accumulated_depreciation_account_id of this AssetType.

        The account for accumulated depreciation of fixed assets of this type  # noqa: E501

        :param accumulated_depreciation_account_id: The accumulated_depreciation_account_id of this AssetType.  # noqa: E501
        :type: str
        """

        self._accumulated_depreciation_account_id = accumulated_depreciation_account_id

    @property
    def book_depreciation_setting(self):
        """Gets the book_depreciation_setting of this AssetType.  # noqa: E501


        :return: The book_depreciation_setting of this AssetType.  # noqa: E501
        :rtype: BookDepreciationSetting
        """
        return self._book_depreciation_setting

    @book_depreciation_setting.setter
    def book_depreciation_setting(self, book_depreciation_setting):
        """Sets the book_depreciation_setting of this AssetType.


        :param book_depreciation_setting: The book_depreciation_setting of this AssetType.  # noqa: E501
        :type: BookDepreciationSetting
        """
        if book_depreciation_setting is None:
            raise ValueError(
                "Invalid value for `book_depreciation_setting`, must not be `None`"
            )  # noqa: E501

        self._book_depreciation_setting = book_depreciation_setting

    @property
    def locks(self):
        """Gets the locks of this AssetType.  # noqa: E501

        All asset types that have accumulated depreciation for any assets that use them are deemed ‘locked’ and cannot be removed.  # noqa: E501

        :return: The locks of this AssetType.  # noqa: E501
        :rtype: int
        """
        return self._locks

    @locks.setter
    def locks(self, locks):
        """Sets the locks of this AssetType.

        All asset types that have accumulated depreciation for any assets that use them are deemed ‘locked’ and cannot be removed.  # noqa: E501

        :param locks: The locks of this AssetType.  # noqa: E501
        :type: int
        """

        self._locks = locks



# coding: utf-8

"""
    Xero Payroll AU API

    This is the Xero Payroll API for orgs in Australia region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class SettingsTrackingCategoriesEmployeeGroups(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"tracking_category_id": "str", "tracking_category_name": "str"}

    attribute_map = {
        "tracking_category_id": "TrackingCategoryID",
        "tracking_category_name": "TrackingCategoryName",
    }

    def __init__(
        self, tracking_category_id=None, tracking_category_name=None
    ):  # noqa: E501
        """SettingsTrackingCategoriesEmployeeGroups - a model defined in OpenAPI"""  # noqa: E501

        self._tracking_category_id = None
        self._tracking_category_name = None
        self.discriminator = None

        if tracking_category_id is not None:
            self.tracking_category_id = tracking_category_id
        if tracking_category_name is not None:
            self.tracking_category_name = tracking_category_name

    @property
    def tracking_category_id(self):
        """Gets the tracking_category_id of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501

        The identifier for the tracking category  # noqa: E501

        :return: The tracking_category_id of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501
        :rtype: str
        """
        return self._tracking_category_id

    @tracking_category_id.setter
    def tracking_category_id(self, tracking_category_id):
        """Sets the tracking_category_id of this SettingsTrackingCategoriesEmployeeGroups.

        The identifier for the tracking category  # noqa: E501

        :param tracking_category_id: The tracking_category_id of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501
        :type: str
        """

        self._tracking_category_id = tracking_category_id

    @property
    def tracking_category_name(self):
        """Gets the tracking_category_name of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501

        Name of the tracking category  # noqa: E501

        :return: The tracking_category_name of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501
        :rtype: str
        """
        return self._tracking_category_name

    @tracking_category_name.setter
    def tracking_category_name(self, tracking_category_name):
        """Sets the tracking_category_name of this SettingsTrackingCategoriesEmployeeGroups.

        Name of the tracking category  # noqa: E501

        :param tracking_category_name: The tracking_category_name of this SettingsTrackingCategoriesEmployeeGroups.  # noqa: E501
        :type: str
        """

        self._tracking_category_name = tracking_category_name



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Setup(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "conversion_date": "ConversionDate",
        "conversion_balances": "list[ConversionBalances]",
        "accounts": "list[Account]",
    }

    attribute_map = {
        "conversion_date": "ConversionDate",
        "conversion_balances": "ConversionBalances",
        "accounts": "Accounts",
    }

    def __init__(
        self, conversion_date=None, conversion_balances=None, accounts=None
    ):  # noqa: E501
        """Setup - a model defined in OpenAPI"""  # noqa: E501

        self._conversion_date = None
        self._conversion_balances = None
        self._accounts = None
        self.discriminator = None

        if conversion_date is not None:
            self.conversion_date = conversion_date
        if conversion_balances is not None:
            self.conversion_balances = conversion_balances
        if accounts is not None:
            self.accounts = accounts

    @property
    def conversion_date(self):
        """Gets the conversion_date of this Setup.  # noqa: E501


        :return: The conversion_date of this Setup.  # noqa: E501
        :rtype: ConversionDate
        """
        return self._conversion_date

    @conversion_date.setter
    def conversion_date(self, conversion_date):
        """Sets the conversion_date of this Setup.


        :param conversion_date: The conversion_date of this Setup.  # noqa: E501
        :type: ConversionDate
        """

        self._conversion_date = conversion_date

    @property
    def conversion_balances(self):
        """Gets the conversion_balances of this Setup.  # noqa: E501

        Balance supplied for each account that has a value as at the conversion date.  # noqa: E501

        :return: The conversion_balances of this Setup.  # noqa: E501
        :rtype: list[ConversionBalances]
        """
        return self._conversion_balances

    @conversion_balances.setter
    def conversion_balances(self, conversion_balances):
        """Sets the conversion_balances of this Setup.

        Balance supplied for each account that has a value as at the conversion date.  # noqa: E501

        :param conversion_balances: The conversion_balances of this Setup.  # noqa: E501
        :type: list[ConversionBalances]
        """

        self._conversion_balances = conversion_balances

    @property
    def accounts(self):
        """Gets the accounts of this Setup.  # noqa: E501


        :return: The accounts of this Setup.  # noqa: E501
        :rtype: list[Account]
        """
        return self._accounts

    @accounts.setter
    def accounts(self, accounts):
        """Sets the accounts of this Setup.


        :param accounts: The accounts of this Setup.  # noqa: E501
        :type: list[Account]
        """

        self._accounts = accounts



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class User(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "user_id": "str",
        "email_address": "str",
        "first_name": "str",
        "last_name": "str",
        "updated_date_utc": "datetime[ms-format]",
        "is_subscriber": "bool",
        "organisation_role": "str",
    }

    attribute_map = {
        "user_id": "UserID",
        "email_address": "EmailAddress",
        "first_name": "FirstName",
        "last_name": "LastName",
        "updated_date_utc": "UpdatedDateUTC",
        "is_subscriber": "IsSubscriber",
        "organisation_role": "OrganisationRole",
    }

    def __init__(
        self,
        user_id=None,
        email_address=None,
        first_name=None,
        last_name=None,
        updated_date_utc=None,
        is_subscriber=None,
        organisation_role=None,
    ):  # noqa: E501
        """User - a model defined in OpenAPI"""  # noqa: E501

        self._user_id = None
        self._email_address = None
        self._first_name = None
        self._last_name = None
        self._updated_date_utc = None
        self._is_subscriber = None
        self._organisation_role = None
        self.discriminator = None

        if user_id is not None:
            self.user_id = user_id
        if email_address is not None:
            self.email_address = email_address
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if is_subscriber is not None:
            self.is_subscriber = is_subscriber
        if organisation_role is not None:
            self.organisation_role = organisation_role

    @property
    def user_id(self):
        """Gets the user_id of this User.  # noqa: E501

        Xero identifier  # noqa: E501

        :return: The user_id of this User.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this User.

        Xero identifier  # noqa: E501

        :param user_id: The user_id of this User.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def email_address(self):
        """Gets the email_address of this User.  # noqa: E501

        Email address of user  # noqa: E501

        :return: The email_address of this User.  # noqa: E501
        :rtype: str
        """
        return self._email_address

    @email_address.setter
    def email_address(self, email_address):
        """Sets the email_address of this User.

        Email address of user  # noqa: E501

        :param email_address: The email_address of this User.  # noqa: E501
        :type: str
        """

        self._email_address = email_address

    @property
    def first_name(self):
        """Gets the first_name of this User.  # noqa: E501

        First name of user  # noqa: E501

        :return: The first_name of this User.  # noqa: E501
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """Sets the first_name of this User.

        First name of user  # noqa: E501

        :param first_name: The first_name of this User.  # noqa: E501
        :type: str
        """

        self._first_name = first_name

    @property
    def last_name(self):
        """Gets the last_name of this User.  # noqa: E501

        Last name of user  # noqa: E501

        :return: The last_name of this User.  # noqa: E501
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """Sets the last_name of this User.

        Last name of user  # noqa: E501

        :param last_name: The last_name of this User.  # noqa: E501
        :type: str
        """

        self._last_name = last_name

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this User.  # noqa: E501

        Timestamp of last change to user  # noqa: E501

        :return: The updated_date_utc of this User.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this User.

        Timestamp of last change to user  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this User.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def is_subscriber(self):
        """Gets the is_subscriber of this User.  # noqa: E501

        Boolean to indicate if user is the subscriber  # noqa: E501

        :return: The is_subscriber of this User.  # noqa: E501
        :rtype: bool
        """
        return self._is_subscriber

    @is_subscriber.setter
    def is_subscriber(self, is_subscriber):
        """Sets the is_subscriber of this User.

        Boolean to indicate if user is the subscriber  # noqa: E501

        :param is_subscriber: The is_subscriber of this User.  # noqa: E501
        :type: bool
        """

        self._is_subscriber = is_subscriber

    @property
    def organisation_role(self):
        """Gets the organisation_role of this User.  # noqa: E501

        User role that defines permissions in Xero and via API (READONLY, INVOICEONLY, STANDARD, FINANCIALADVISER, etc)  # noqa: E501

        :return: The organisation_role of this User.  # noqa: E501
        :rtype: str
        """
        return self._organisation_role

    @organisation_role.setter
    def organisation_role(self, organisation_role):
        """Sets the organisation_role of this User.

        User role that defines permissions in Xero and via API (READONLY, INVOICEONLY, STANDARD, FINANCIALADVISER, etc)  # noqa: E501

        :param organisation_role: The organisation_role of this User.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "READONLY",
            "INVOICEONLY",
            "STANDARD",
            "FINANCIALADVISER",
            "MANAGEDCLIENT",
            "CASHBOOKCLIENT",
            "UNKNOWN",
            "None",
        ]  # noqa: E501

        if organisation_role:
            if organisation_role not in allowed_values:
                raise ValueError(
                    "Invalid value for `organisation_role` ({0}), must be one of {1}".format(  # noqa: E501
                        organisation_role, allowed_values
                    )
                )

        self._organisation_role = organisation_role



# coding: utf-8

"""
    Xero Assets API

    The Assets API exposes fixed asset related functions of the Xero Accounting application and can be used for a variety of purposes such as creating assets, retrieving asset valuations etc.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""

"""
    OpenAPI spec version: 2.10.4
"""

import importlib
import re  # noqa: F401

from xero_python import exceptions
from xero_python.api_client import ApiClient, ModelFinder

try:
    from .exception_handler import translate_status_exception
except ImportError:
    translate_status_exception = exceptions.translate_status_exception


class empty:
    """ empty object to mark optional parameter not set """


class AssetApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    base_url = "https://api.xero.com/assets.xro/1.0"
    models_module = importlib.import_module("xero_python.assets.models")

    def __init__(self, api_client=None, base_url=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.base_url = base_url or self.base_url

    def get_resource_url(self, resource_path):
        """
        Combine API base url with resource specific path
        :param str resource_path: API endpoint specific path
        :return: str full resource path
        """
        return self.base_url + resource_path

    def get_model_finder(self):
        return ModelFinder(self.models_module)

    def create_asset(
        self,
        xero_tenant_id,
        asset,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """adds a fixed asset  # noqa: E501
        OAuth2 scope: assets
        Adds an asset to the system  # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param Asset asset: Fixed asset you are creating (required)
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: Asset
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `create_asset`"
            )
        # verify the required parameter 'asset' is set
        if asset is None:
            raise ValueError(
                "Missing the required parameter `asset` " "when calling `create_asset`"
            )

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = asset
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/Assets")

        try:
            return self.api_client.call_api(
                url,
                "POST",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="Asset",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "create_asset")

    def create_asset_type(
        self,
        xero_tenant_id,
        asset_type=empty,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """adds a fixed asset type  # noqa: E501
        OAuth2 scope: assets
        Adds an fixed asset type to the system  # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param AssetType asset_type: Asset type to add
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: AssetType
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `create_asset_type`"
            )

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = asset_type
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/AssetTypes")

        try:
            return self.api_client.call_api(
                url,
                "POST",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="AssetType",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "create_asset_type")

    def get_asset_by_id(
        self,
        xero_tenant_id,
        id,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Retrieves fixed asset by id  # noqa: E501
        OAuth2 scope: assets.read
        By passing in the appropriate asset id, you can search for a specific fixed asset in the system   # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param str id: fixed asset id for single object (required)
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: Asset
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `get_asset_by_id`"
            )
        # verify the required parameter 'id' is set
        if id is None:
            raise ValueError(
                "Missing the required parameter `id` " "when calling `get_asset_by_id`"
            )

        collection_formats = {}
        path_params = {
            "id": id,
        }

        query_params = []

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/Assets/{id}")

        try:
            return self.api_client.call_api(
                url,
                "GET",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="Asset",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "get_asset_by_id")

    def get_asset_settings(
        self,
        xero_tenant_id,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """searches fixed asset settings  # noqa: E501
        OAuth2 scope: assets.read
        By passing in the appropriate options, you can search for available fixed asset types in the system  # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: Setting
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `get_asset_settings`"
            )

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/Settings")

        try:
            return self.api_client.call_api(
                url,
                "GET",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="Setting",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "get_asset_settings")

    def get_asset_types(
        self,
        xero_tenant_id,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """searches fixed asset types  # noqa: E501
        OAuth2 scope: assets.read
        By passing in the appropriate options, you can search for available fixed asset types in the system  # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: list[AssetType]
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `get_asset_types`"
            )

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/AssetTypes")

        try:
            return self.api_client.call_api(
                url,
                "GET",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="list[AssetType]",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "get_asset_types")

    def get_assets(
        self,
        xero_tenant_id,
        status,
        page=empty,
        page_size=empty,
        order_by=empty,
        sort_direction=empty,
        filter_by=empty,
        _return_http_data_only=True,
        _preload_content=True,
        _request_timeout=None,
    ):
        """searches fixed asset  # noqa: E501
        OAuth2 scope: assets.read
        By passing in the appropriate options, you can search for available fixed asset in the system  # noqa: E501
        :param str xero_tenant_id: Xero identifier for Tenant (required)
        :param AssetStatusQueryParam status: Required when retrieving a collection of assets. See Asset Status Codes (required)
        :param int page: Results are paged. This specifies which page of the results to return. The default page is 1.
        :param int page_size: The number of records returned per page. By default the number of records returned is 10.
        :param str order_by: Requests can be ordered by AssetType, AssetName, AssetNumber, PurchaseDate and PurchasePrice. If the asset status is DISPOSED it also allows DisposalDate and DisposalPrice.
        :param str sort_direction: ASC or DESC
        :param str filter_by: A string that can be used to filter the list to only return assets containing the text. Checks it against the AssetName, AssetNumber, Description and AssetTypeName fields.
        :param bool _return_http_data_only: return received data only
        :param bool _preload_content: load received data in models
        :param bool _request_timeout: maximum wait time for response
        :return: Assets
        """

        # verify the required parameter 'xero_tenant_id' is set
        if xero_tenant_id is None:
            raise ValueError(
                "Missing the required parameter `xero_tenant_id` "
                "when calling `get_assets`"
            )
        # verify the required parameter 'status' is set
        if status is None:
            raise ValueError(
                "Missing the required parameter `status` " "when calling `get_assets`"
            )

        collection_formats = {}
        path_params = {}

        query_params = [
            ("status", status),
        ]

        if page is not empty:
            query_params.append(("page", page))

        if page_size is not empty:
            query_params.append(("pageSize", page_size))

        if order_by is not empty:
            query_params.append(("orderBy", order_by))

        if sort_direction is not empty:
            query_params.append(("sortDirection", sort_direction))

        if filter_by is not empty:
            query_params.append(("filterBy", filter_by))

        header_params = {
            "xero-tenant-id": xero_tenant_id,
        }

        local_var_files = {}
        form_params = []

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = ["OAuth2"]
        url = self.get_resource_url("/Assets")

        try:
            return self.api_client.call_api(
                url,
                "GET",
                path_params,
                query_params,
                header_params,
                body=body_params,
                post_params=form_params,
                files=local_var_files,
                response_type="Assets",
                response_model_finder=self.get_model_finder(),
                auth_settings=auth_settings,
                _return_http_data_only=_return_http_data_only,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                collection_formats=collection_formats,
            )
        except exceptions.HTTPStatusException as error:
            raise translate_status_exception(error, self, "get_assets")



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class DeductionLine(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "deduction_type_id": "str",
        "amount": "float",
        "subject_to_tax": "bool",
        "percentage": "float",
    }

    attribute_map = {
        "deduction_type_id": "deductionTypeID",
        "amount": "amount",
        "subject_to_tax": "subjectToTax",
        "percentage": "percentage",
    }

    def __init__(
        self, deduction_type_id=None, amount=None, subject_to_tax=None, percentage=None
    ):  # noqa: E501
        """DeductionLine - a model defined in OpenAPI"""  # noqa: E501

        self._deduction_type_id = None
        self._amount = None
        self._subject_to_tax = None
        self._percentage = None
        self.discriminator = None

        if deduction_type_id is not None:
            self.deduction_type_id = deduction_type_id
        if amount is not None:
            self.amount = amount
        if subject_to_tax is not None:
            self.subject_to_tax = subject_to_tax
        if percentage is not None:
            self.percentage = percentage

    @property
    def deduction_type_id(self):
        """Gets the deduction_type_id of this DeductionLine.  # noqa: E501

        Xero identifier for payroll deduction  # noqa: E501

        :return: The deduction_type_id of this DeductionLine.  # noqa: E501
        :rtype: str
        """
        return self._deduction_type_id

    @deduction_type_id.setter
    def deduction_type_id(self, deduction_type_id):
        """Sets the deduction_type_id of this DeductionLine.

        Xero identifier for payroll deduction  # noqa: E501

        :param deduction_type_id: The deduction_type_id of this DeductionLine.  # noqa: E501
        :type: str
        """

        self._deduction_type_id = deduction_type_id

    @property
    def amount(self):
        """Gets the amount of this DeductionLine.  # noqa: E501

        The amount of the deduction line  # noqa: E501

        :return: The amount of this DeductionLine.  # noqa: E501
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this DeductionLine.

        The amount of the deduction line  # noqa: E501

        :param amount: The amount of this DeductionLine.  # noqa: E501
        :type: float
        """

        self._amount = amount

    @property
    def subject_to_tax(self):
        """Gets the subject_to_tax of this DeductionLine.  # noqa: E501

        Identifies if the deduction is subject to tax  # noqa: E501

        :return: The subject_to_tax of this DeductionLine.  # noqa: E501
        :rtype: bool
        """
        return self._subject_to_tax

    @subject_to_tax.setter
    def subject_to_tax(self, subject_to_tax):
        """Sets the subject_to_tax of this DeductionLine.

        Identifies if the deduction is subject to tax  # noqa: E501

        :param subject_to_tax: The subject_to_tax of this DeductionLine.  # noqa: E501
        :type: bool
        """

        self._subject_to_tax = subject_to_tax

    @property
    def percentage(self):
        """Gets the percentage of this DeductionLine.  # noqa: E501

        Deduction rate percentage  # noqa: E501

        :return: The percentage of this DeductionLine.  # noqa: E501
        :rtype: float
        """
        return self._percentage

    @percentage.setter
    def percentage(self, percentage):
        """Sets the percentage of this DeductionLine.

        Deduction rate percentage  # noqa: E501

        :param percentage: The percentage of this DeductionLine.  # noqa: E501
        :type: float
        """

        self._percentage = percentage



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Employment(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "payroll_calendar_id": "str",
        "pay_run_calendar_id": "str",
        "start_date": "date",
    }

    attribute_map = {
        "payroll_calendar_id": "payrollCalendarID",
        "pay_run_calendar_id": "payRunCalendarID",
        "start_date": "startDate",
    }

    def __init__(
        self, payroll_calendar_id=None, pay_run_calendar_id=None, start_date=None
    ):  # noqa: E501
        """Employment - a model defined in OpenAPI"""  # noqa: E501

        self._payroll_calendar_id = None
        self._pay_run_calendar_id = None
        self._start_date = None
        self.discriminator = None

        if payroll_calendar_id is not None:
            self.payroll_calendar_id = payroll_calendar_id
        if pay_run_calendar_id is not None:
            self.pay_run_calendar_id = pay_run_calendar_id
        if start_date is not None:
            self.start_date = start_date

    @property
    def payroll_calendar_id(self):
        """Gets the payroll_calendar_id of this Employment.  # noqa: E501

        Xero unique identifier for the payroll calendar of the employee  # noqa: E501

        :return: The payroll_calendar_id of this Employment.  # noqa: E501
        :rtype: str
        """
        return self._payroll_calendar_id

    @payroll_calendar_id.setter
    def payroll_calendar_id(self, payroll_calendar_id):
        """Sets the payroll_calendar_id of this Employment.

        Xero unique identifier for the payroll calendar of the employee  # noqa: E501

        :param payroll_calendar_id: The payroll_calendar_id of this Employment.  # noqa: E501
        :type: str
        """

        self._payroll_calendar_id = payroll_calendar_id

    @property
    def pay_run_calendar_id(self):
        """Gets the pay_run_calendar_id of this Employment.  # noqa: E501

        Xero unique identifier for the payrun calendar for the employee (Deprecated in version 1.1.6)  # noqa: E501

        :return: The pay_run_calendar_id of this Employment.  # noqa: E501
        :rtype: str
        """
        return self._pay_run_calendar_id

    @pay_run_calendar_id.setter
    def pay_run_calendar_id(self, pay_run_calendar_id):
        """Sets the pay_run_calendar_id of this Employment.

        Xero unique identifier for the payrun calendar for the employee (Deprecated in version 1.1.6)  # noqa: E501

        :param pay_run_calendar_id: The pay_run_calendar_id of this Employment.  # noqa: E501
        :type: str
        """

        self._pay_run_calendar_id = pay_run_calendar_id

    @property
    def start_date(self):
        """Gets the start_date of this Employment.  # noqa: E501

        Start date of the employment (YYYY-MM-DD)  # noqa: E501

        :return: The start_date of this Employment.  # noqa: E501
        :rtype: date
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this Employment.

        Start date of the employment (YYYY-MM-DD)  # noqa: E501

        :param start_date: The start_date of this Employment.  # noqa: E501
        :type: date
        """

        self._start_date = start_date



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class OnlineInvoice(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"online_invoice_url": "str"}

    attribute_map = {"online_invoice_url": "OnlineInvoiceUrl"}

    def __init__(self, online_invoice_url=None):  # noqa: E501
        """OnlineInvoice - a model defined in OpenAPI"""  # noqa: E501

        self._online_invoice_url = None
        self.discriminator = None

        if online_invoice_url is not None:
            self.online_invoice_url = online_invoice_url

    @property
    def online_invoice_url(self):
        """Gets the online_invoice_url of this OnlineInvoice.  # noqa: E501

        the URL to an online invoice  # noqa: E501

        :return: The online_invoice_url of this OnlineInvoice.  # noqa: E501
        :rtype: str
        """
        return self._online_invoice_url

    @online_invoice_url.setter
    def online_invoice_url(self, online_invoice_url):
        """Sets the online_invoice_url of this OnlineInvoice.

        the URL to an online invoice  # noqa: E501

        :param online_invoice_url: The online_invoice_url of this OnlineInvoice.  # noqa: E501
        :type: str
        """

        self._online_invoice_url = online_invoice_url



# coding: utf-8

"""
    Xero Accounting API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class Employee(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "employee_id": "str",
        "status": "str",
        "first_name": "str",
        "last_name": "str",
        "external_link": "ExternalLink",
        "updated_date_utc": "datetime[ms-format]",
        "status_attribute_string": "str",
        "validation_errors": "list[ValidationError]",
    }

    attribute_map = {
        "employee_id": "EmployeeID",
        "status": "Status",
        "first_name": "FirstName",
        "last_name": "LastName",
        "external_link": "ExternalLink",
        "updated_date_utc": "UpdatedDateUTC",
        "status_attribute_string": "StatusAttributeString",
        "validation_errors": "ValidationErrors",
    }

    def __init__(
        self,
        employee_id=None,
        status=None,
        first_name=None,
        last_name=None,
        external_link=None,
        updated_date_utc=None,
        status_attribute_string=None,
        validation_errors=None,
    ):  # noqa: E501
        """Employee - a model defined in OpenAPI"""  # noqa: E501

        self._employee_id = None
        self._status = None
        self._first_name = None
        self._last_name = None
        self._external_link = None
        self._updated_date_utc = None
        self._status_attribute_string = None
        self._validation_errors = None
        self.discriminator = None

        if employee_id is not None:
            self.employee_id = employee_id
        if status is not None:
            self.status = status
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if external_link is not None:
            self.external_link = external_link
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if status_attribute_string is not None:
            self.status_attribute_string = status_attribute_string
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def employee_id(self):
        """Gets the employee_id of this Employee.  # noqa: E501

        The Xero identifier for an employee e.g. 297c2dc5-cc47-4afd-8ec8-74990b8761e9  # noqa: E501

        :return: The employee_id of this Employee.  # noqa: E501
        :rtype: str
        """
        return self._employee_id

    @employee_id.setter
    def employee_id(self, employee_id):
        """Sets the employee_id of this Employee.

        The Xero identifier for an employee e.g. 297c2dc5-cc47-4afd-8ec8-74990b8761e9  # noqa: E501

        :param employee_id: The employee_id of this Employee.  # noqa: E501
        :type: str
        """

        self._employee_id = employee_id

    @property
    def status(self):
        """Gets the status of this Employee.  # noqa: E501

        Current status of an employee – see contact status types  # noqa: E501

        :return: The status of this Employee.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Employee.

        Current status of an employee – see contact status types  # noqa: E501

        :param status: The status of this Employee.  # noqa: E501
        :type: str
        """
        allowed_values = [
            "ACTIVE",
            "ARCHIVED",
            "GDPRREQUEST",
            "DELETED",
            "None",
        ]  # noqa: E501

        if status:
            if status not in allowed_values:
                raise ValueError(
                    "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                        status, allowed_values
                    )
                )

        self._status = status

    @property
    def first_name(self):
        """Gets the first_name of this Employee.  # noqa: E501

        First name of an employee (max length = 255)  # noqa: E501

        :return: The first_name of this Employee.  # noqa: E501
        :rtype: str
        """
        return self._first_name

    @first_name.setter
    def first_name(self, first_name):
        """Sets the first_name of this Employee.

        First name of an employee (max length = 255)  # noqa: E501

        :param first_name: The first_name of this Employee.  # noqa: E501
        :type: str
        """
        if first_name is not None and len(first_name) > 255:
            raise ValueError(
                "Invalid value for `first_name`, "
                "length must be less than or equal to `255`"
            )  # noqa: E501

        self._first_name = first_name

    @property
    def last_name(self):
        """Gets the last_name of this Employee.  # noqa: E501

        Last name of an employee (max length = 255)  # noqa: E501

        :return: The last_name of this Employee.  # noqa: E501
        :rtype: str
        """
        return self._last_name

    @last_name.setter
    def last_name(self, last_name):
        """Sets the last_name of this Employee.

        Last name of an employee (max length = 255)  # noqa: E501

        :param last_name: The last_name of this Employee.  # noqa: E501
        :type: str
        """
        if last_name is not None and len(last_name) > 255:
            raise ValueError(
                "Invalid value for `last_name`, "
                "length must be less than or equal to `255`"
            )  # noqa: E501

        self._last_name = last_name

    @property
    def external_link(self):
        """Gets the external_link of this Employee.  # noqa: E501


        :return: The external_link of this Employee.  # noqa: E501
        :rtype: ExternalLink
        """
        return self._external_link

    @external_link.setter
    def external_link(self, external_link):
        """Sets the external_link of this Employee.


        :param external_link: The external_link of this Employee.  # noqa: E501
        :type: ExternalLink
        """

        self._external_link = external_link

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this Employee.  # noqa: E501


        :return: The updated_date_utc of this Employee.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this Employee.


        :param updated_date_utc: The updated_date_utc of this Employee.  # noqa: E501
        :type: datetime
        """

        self._updated_date_utc = updated_date_utc

    @property
    def status_attribute_string(self):
        """Gets the status_attribute_string of this Employee.  # noqa: E501

        A string to indicate if a invoice status  # noqa: E501

        :return: The status_attribute_string of this Employee.  # noqa: E501
        :rtype: str
        """
        return self._status_attribute_string

    @status_attribute_string.setter
    def status_attribute_string(self, status_attribute_string):
        """Sets the status_attribute_string of this Employee.

        A string to indicate if a invoice status  # noqa: E501

        :param status_attribute_string: The status_attribute_string of this Employee.  # noqa: E501
        :type: str
        """

        self._status_attribute_string = status_attribute_string

    @property
    def validation_errors(self):
        """Gets the validation_errors of this Employee.  # noqa: E501

        Displays array of validation error messages from the API  # noqa: E501

        :return: The validation_errors of this Employee.  # noqa: E501
        :rtype: list[ValidationError]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this Employee.

        Displays array of validation error messages from the API  # noqa: E501

        :param validation_errors: The validation_errors of this Employee.  # noqa: E501
        :type: list[ValidationError]
        """

        self._validation_errors = validation_errors



# coding: utf-8

"""
    Xero Payroll UK

    This is the Xero Payroll API for orgs in the UK region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EmployeeStatutorySickLeave(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "statutory_leave_id": "str",
        "employee_id": "str",
        "leave_type_id": "str",
        "start_date": "date",
        "end_date": "date",
        "type": "str",
        "status": "str",
        "work_pattern": "list[str]",
        "is_pregnancy_related": "bool",
        "sufficient_notice": "bool",
        "is_entitled": "bool",
        "entitlement_weeks_requested": "float",
        "entitlement_weeks_qualified": "float",
        "entitlement_weeks_remaining": "float",
        "overlaps_with_other_leave": "bool",
        "entitlement_failure_reasons": "list[str]",
    }

    attribute_map = {
        "statutory_leave_id": "statutoryLeaveID",
        "employee_id": "employeeID",
        "leave_type_id": "leaveTypeID",
        "start_date": "startDate",
        "end_date": "endDate",
        "type": "type",
        "status": "status",
        "work_pattern": "workPattern",
        "is_pregnancy_related": "isPregnancyRelated",
        "sufficient_notice": "sufficientNotice",
        "is_entitled": "isEntitled",
        "entitlement_weeks_requested": "entitlementWeeksRequested",
        "entitlement_weeks_qualified": "entitlementWeeksQualified",
        "entitlement_weeks_remaining": "entitlementWeeksRemaining",
        "overlaps_with_other_leave": "overlapsWithOtherLeave",
        "entitlement_failure_reasons": "entitlementFailureReasons",
    }

    def __init__(
        self,
        statutory_leave_id=None,
        employee_id=None,
        leave_type_id=None,
        start_date=None,
        end_date=None,
        type=None,
        status=None,
        work_pattern=None,
        is_pregnancy_related=None,
        sufficient_notice=None,
        is_entitled=None,
        entitlement_weeks_requested=None,
        entitlement_weeks_qualified=None,
        entitlement_weeks_remaining=None,
        overlaps_with_other_leave=None,
        entitlement_failure_reasons=None,
    ):  # noqa: E501
        """EmployeeStatutorySickLeave - a model defined in OpenAPI"""  # noqa: E501

        self._statutory_leave_id = None
        self._employee_id = None
        self._leave_type_id = None
        self._start_date = None
        self._end_date = None
        self._type = None
        self._status = None
        self._work_pattern = None
        self._is_pregnancy_related = None
        self._sufficient_notice = None
        self._is_entitled = None
        self._entitlement_weeks_requested = None
        self._entitlement_weeks_qualified = None
        self._entitlement_weeks_remaining = None
        self._overlaps_with_other_leave = None
        self._entitlement_failure_reasons = None
        self.discriminator = None

        if statutory_leave_id is not None:
            self.statutory_leave_id = statutory_leave_id
        self.employee_id = employee_id
        self.leave_type_id = leave_type_id
        self.start_date = start_date
        self.end_date = end_date
        if type is not None:
            self.type = type
        if status is not None:
            self.status = status
        self.work_pattern = work_pattern
        self.is_pregnancy_related = is_pregnancy_related
        self.sufficient_notice = sufficient_notice
        if is_entitled is not None:
            self.is_entitled = is_entitled
        if entitlement_weeks_requested is not None:
            self.entitlement_weeks_requested = entitlement_weeks_requested
        if entitlement_weeks_qualified is not None:
            self.entitlement_weeks_qualified = entitlement_weeks_qualified
        if entitlement_weeks_remaining is not None:
            self.entitlement_weeks_remaining = entitlement_weeks_remaining
        if overlaps_with_other_leave is not None:
            self.overlaps_with_other_leave = overlaps_with_other_leave
        if entitlement_failure_reasons is not None:
            self.entitlement_failure_reasons = entitlement_failure_reasons

    @property
    def statutory_leave_id(self):
        """Gets the statutory_leave_id of this EmployeeStatutorySickLeave.  # noqa: E501

        The unique identifier (guid) of a statutory leave  # noqa: E501

        :return: The statutory_leave_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: str
        """
        return self._statutory_leave_id

    @statutory_leave_id.setter
    def statutory_leave_id(self, statutory_leave_id):
        """Sets the statutory_leave_id of this EmployeeStatutorySickLeave.

        The unique identifier (guid) of a statutory leave  # noqa: E501

        :param statutory_leave_id: The statutory_leave_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: str
        """

        self._statutory_leave_id = statutory_leave_id

    @property
    def employee_id(self):
        """Gets the employee_id of this EmployeeStatutorySickLeave.  # noqa: E501

        The unique identifier (guid) of the employee  # noqa: E501

        :return: The employee_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: str
        """
        return self._employee_id

    @employee_id.setter
    def employee_id(self, employee_id):
        """Sets the employee_id of this EmployeeStatutorySickLeave.

        The unique identifier (guid) of the employee  # noqa: E501

        :param employee_id: The employee_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: str
        """
        if employee_id is None:
            raise ValueError(
                "Invalid value for `employee_id`, must not be `None`"
            )  # noqa: E501

        self._employee_id = employee_id

    @property
    def leave_type_id(self):
        """Gets the leave_type_id of this EmployeeStatutorySickLeave.  # noqa: E501

        The unique identifier (guid) of the \"Statutory Sick Leave (non-pensionable)\" pay item  # noqa: E501

        :return: The leave_type_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: str
        """
        return self._leave_type_id

    @leave_type_id.setter
    def leave_type_id(self, leave_type_id):
        """Sets the leave_type_id of this EmployeeStatutorySickLeave.

        The unique identifier (guid) of the \"Statutory Sick Leave (non-pensionable)\" pay item  # noqa: E501

        :param leave_type_id: The leave_type_id of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: str
        """
        if leave_type_id is None:
            raise ValueError(
                "Invalid value for `leave_type_id`, must not be `None`"
            )  # noqa: E501

        self._leave_type_id = leave_type_id

    @property
    def start_date(self):
        """Gets the start_date of this EmployeeStatutorySickLeave.  # noqa: E501

        The date when the leave starts  # noqa: E501

        :return: The start_date of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: date
        """
        return self._start_date

    @start_date.setter
    def start_date(self, start_date):
        """Sets the start_date of this EmployeeStatutorySickLeave.

        The date when the leave starts  # noqa: E501

        :param start_date: The start_date of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: date
        """
        if start_date is None:
            raise ValueError(
                "Invalid value for `start_date`, must not be `None`"
            )  # noqa: E501

        self._start_date = start_date

    @property
    def end_date(self):
        """Gets the end_date of this EmployeeStatutorySickLeave.  # noqa: E501

        The date when the leave ends  # noqa: E501

        :return: The end_date of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: date
        """
        return self._end_date

    @end_date.setter
    def end_date(self, end_date):
        """Sets the end_date of this EmployeeStatutorySickLeave.

        The date when the leave ends  # noqa: E501

        :param end_date: The end_date of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: date
        """
        if end_date is None:
            raise ValueError(
                "Invalid value for `end_date`, must not be `None`"
            )  # noqa: E501

        self._end_date = end_date

    @property
    def type(self):
        """Gets the type of this EmployeeStatutorySickLeave.  # noqa: E501

        the type of statutory leave  # noqa: E501

        :return: The type of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this EmployeeStatutorySickLeave.

        the type of statutory leave  # noqa: E501

        :param type: The type of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def status(self):
        """Gets the status of this EmployeeStatutorySickLeave.  # noqa: E501

        the type of statutory leave  # noqa: E501

        :return: The status of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this EmployeeStatutorySickLeave.

        the type of statutory leave  # noqa: E501

        :param status: The status of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def work_pattern(self):
        """Gets the work_pattern of this EmployeeStatutorySickLeave.  # noqa: E501

        The days of the work week the employee is scheduled to work at the time the leave is taken  # noqa: E501

        :return: The work_pattern of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: list[str]
        """
        return self._work_pattern

    @work_pattern.setter
    def work_pattern(self, work_pattern):
        """Sets the work_pattern of this EmployeeStatutorySickLeave.

        The days of the work week the employee is scheduled to work at the time the leave is taken  # noqa: E501

        :param work_pattern: The work_pattern of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: list[str]
        """
        if work_pattern is None:
            raise ValueError(
                "Invalid value for `work_pattern`, must not be `None`"
            )  # noqa: E501

        self._work_pattern = work_pattern

    @property
    def is_pregnancy_related(self):
        """Gets the is_pregnancy_related of this EmployeeStatutorySickLeave.  # noqa: E501

        Whether the sick leave was pregnancy related  # noqa: E501

        :return: The is_pregnancy_related of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: bool
        """
        return self._is_pregnancy_related

    @is_pregnancy_related.setter
    def is_pregnancy_related(self, is_pregnancy_related):
        """Sets the is_pregnancy_related of this EmployeeStatutorySickLeave.

        Whether the sick leave was pregnancy related  # noqa: E501

        :param is_pregnancy_related: The is_pregnancy_related of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: bool
        """
        if is_pregnancy_related is None:
            raise ValueError(
                "Invalid value for `is_pregnancy_related`, must not be `None`"
            )  # noqa: E501

        self._is_pregnancy_related = is_pregnancy_related

    @property
    def sufficient_notice(self):
        """Gets the sufficient_notice of this EmployeeStatutorySickLeave.  # noqa: E501

        Whether the employee provided sufficient notice and documentation as required by the employer supporting the sick leave request  # noqa: E501

        :return: The sufficient_notice of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: bool
        """
        return self._sufficient_notice

    @sufficient_notice.setter
    def sufficient_notice(self, sufficient_notice):
        """Sets the sufficient_notice of this EmployeeStatutorySickLeave.

        Whether the employee provided sufficient notice and documentation as required by the employer supporting the sick leave request  # noqa: E501

        :param sufficient_notice: The sufficient_notice of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: bool
        """
        if sufficient_notice is None:
            raise ValueError(
                "Invalid value for `sufficient_notice`, must not be `None`"
            )  # noqa: E501

        self._sufficient_notice = sufficient_notice

    @property
    def is_entitled(self):
        """Gets the is_entitled of this EmployeeStatutorySickLeave.  # noqa: E501

        Whether the leave was entitled to receive payment  # noqa: E501

        :return: The is_entitled of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: bool
        """
        return self._is_entitled

    @is_entitled.setter
    def is_entitled(self, is_entitled):
        """Sets the is_entitled of this EmployeeStatutorySickLeave.

        Whether the leave was entitled to receive payment  # noqa: E501

        :param is_entitled: The is_entitled of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: bool
        """

        self._is_entitled = is_entitled

    @property
    def entitlement_weeks_requested(self):
        """Gets the entitlement_weeks_requested of this EmployeeStatutorySickLeave.  # noqa: E501

        The amount of requested time (in weeks)  # noqa: E501

        :return: The entitlement_weeks_requested of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: float
        """
        return self._entitlement_weeks_requested

    @entitlement_weeks_requested.setter
    def entitlement_weeks_requested(self, entitlement_weeks_requested):
        """Sets the entitlement_weeks_requested of this EmployeeStatutorySickLeave.

        The amount of requested time (in weeks)  # noqa: E501

        :param entitlement_weeks_requested: The entitlement_weeks_requested of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: float
        """

        self._entitlement_weeks_requested = entitlement_weeks_requested

    @property
    def entitlement_weeks_qualified(self):
        """Gets the entitlement_weeks_qualified of this EmployeeStatutorySickLeave.  # noqa: E501

        The amount of statutory sick leave time off (in weeks) that is available to take at the time the leave was requested  # noqa: E501

        :return: The entitlement_weeks_qualified of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: float
        """
        return self._entitlement_weeks_qualified

    @entitlement_weeks_qualified.setter
    def entitlement_weeks_qualified(self, entitlement_weeks_qualified):
        """Sets the entitlement_weeks_qualified of this EmployeeStatutorySickLeave.

        The amount of statutory sick leave time off (in weeks) that is available to take at the time the leave was requested  # noqa: E501

        :param entitlement_weeks_qualified: The entitlement_weeks_qualified of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: float
        """

        self._entitlement_weeks_qualified = entitlement_weeks_qualified

    @property
    def entitlement_weeks_remaining(self):
        """Gets the entitlement_weeks_remaining of this EmployeeStatutorySickLeave.  # noqa: E501

        A calculated amount of time (in weeks) that remains for the statutory sick leave period  # noqa: E501

        :return: The entitlement_weeks_remaining of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: float
        """
        return self._entitlement_weeks_remaining

    @entitlement_weeks_remaining.setter
    def entitlement_weeks_remaining(self, entitlement_weeks_remaining):
        """Sets the entitlement_weeks_remaining of this EmployeeStatutorySickLeave.

        A calculated amount of time (in weeks) that remains for the statutory sick leave period  # noqa: E501

        :param entitlement_weeks_remaining: The entitlement_weeks_remaining of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: float
        """

        self._entitlement_weeks_remaining = entitlement_weeks_remaining

    @property
    def overlaps_with_other_leave(self):
        """Gets the overlaps_with_other_leave of this EmployeeStatutorySickLeave.  # noqa: E501

        Whether another leave (Paternity, Shared Parental specifically) occurs during the requested leave's period. While this is allowed it could affect payment amounts  # noqa: E501

        :return: The overlaps_with_other_leave of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: bool
        """
        return self._overlaps_with_other_leave

    @overlaps_with_other_leave.setter
    def overlaps_with_other_leave(self, overlaps_with_other_leave):
        """Sets the overlaps_with_other_leave of this EmployeeStatutorySickLeave.

        Whether another leave (Paternity, Shared Parental specifically) occurs during the requested leave's period. While this is allowed it could affect payment amounts  # noqa: E501

        :param overlaps_with_other_leave: The overlaps_with_other_leave of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: bool
        """

        self._overlaps_with_other_leave = overlaps_with_other_leave

    @property
    def entitlement_failure_reasons(self):
        """Gets the entitlement_failure_reasons of this EmployeeStatutorySickLeave.  # noqa: E501

        If the leave requested was considered \"not entitled\", the reasons why are listed here.  # noqa: E501

        :return: The entitlement_failure_reasons of this EmployeeStatutorySickLeave.  # noqa: E501
        :rtype: list[str]
        """
        return self._entitlement_failure_reasons

    @entitlement_failure_reasons.setter
    def entitlement_failure_reasons(self, entitlement_failure_reasons):
        """Sets the entitlement_failure_reasons of this EmployeeStatutorySickLeave.

        If the leave requested was considered \"not entitled\", the reasons why are listed here.  # noqa: E501

        :param entitlement_failure_reasons: The entitlement_failure_reasons of this EmployeeStatutorySickLeave.  # noqa: E501
        :type: list[str]
        """
        allowed_values = [
            "UnableToCalculateAwe",
            "AweLowerThanLel",
            "NotQualifiedInPreviousPiw",
            "ExceededMaximumEntitlementWeeksOfSsp",
            "ExceededMaximumDurationOfPiw",
            "SufficientNoticeNotGiven",
            "None",
        ]  # noqa: E501
        if not set(entitlement_failure_reasons).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `entitlement_failure_reasons` [{0}], must be a subset of [{1}]".format(  # noqa: E501
                    ", ".join(
                        map(str, set(entitlement_failure_reasons) - set(allowed_values))
                    ),  # noqa: E501
                    ", ".join(map(str, allowed_values)),
                )
            )

        self._entitlement_failure_reasons = entitlement_failure_reasons



# coding: utf-8

"""
    Xero Payroll NZ

    This is the Xero Payroll API for orgs in the NZ region.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class EmployeeOpeningBalance(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "period_end_date": "date",
        "days_paid": "int",
        "unpaid_weeks": "int",
        "gross_earnings": "float",
    }

    attribute_map = {
        "period_end_date": "periodEndDate",
        "days_paid": "daysPaid",
        "unpaid_weeks": "unpaidWeeks",
        "gross_earnings": "grossEarnings",
    }

    def __init__(
        self,
        period_end_date=None,
        days_paid=None,
        unpaid_weeks=None,
        gross_earnings=None,
    ):  # noqa: E501
        """EmployeeOpeningBalance - a model defined in OpenAPI"""  # noqa: E501

        self._period_end_date = None
        self._days_paid = None
        self._unpaid_weeks = None
        self._gross_earnings = None
        self.discriminator = None

        if period_end_date is not None:
            self.period_end_date = period_end_date
        if days_paid is not None:
            self.days_paid = days_paid
        if unpaid_weeks is not None:
            self.unpaid_weeks = unpaid_weeks
        if gross_earnings is not None:
            self.gross_earnings = gross_earnings

    @property
    def period_end_date(self):
        """Gets the period_end_date of this EmployeeOpeningBalance.  # noqa: E501

        The opening balance period end date.  # noqa: E501

        :return: The period_end_date of this EmployeeOpeningBalance.  # noqa: E501
        :rtype: date
        """
        return self._period_end_date

    @period_end_date.setter
    def period_end_date(self, period_end_date):
        """Sets the period_end_date of this EmployeeOpeningBalance.

        The opening balance period end date.  # noqa: E501

        :param period_end_date: The period_end_date of this EmployeeOpeningBalance.  # noqa: E501
        :type: date
        """

        self._period_end_date = period_end_date

    @property
    def days_paid(self):
        """Gets the days_paid of this EmployeeOpeningBalance.  # noqa: E501

        The paid number of days.  # noqa: E501

        :return: The days_paid of this EmployeeOpeningBalance.  # noqa: E501
        :rtype: int
        """
        return self._days_paid

    @days_paid.setter
    def days_paid(self, days_paid):
        """Sets the days_paid of this EmployeeOpeningBalance.

        The paid number of days.  # noqa: E501

        :param days_paid: The days_paid of this EmployeeOpeningBalance.  # noqa: E501
        :type: int
        """

        self._days_paid = days_paid

    @property
    def unpaid_weeks(self):
        """Gets the unpaid_weeks of this EmployeeOpeningBalance.  # noqa: E501

        The number of unpaid weeks.  # noqa: E501

        :return: The unpaid_weeks of this EmployeeOpeningBalance.  # noqa: E501
        :rtype: int
        """
        return self._unpaid_weeks

    @unpaid_weeks.setter
    def unpaid_weeks(self, unpaid_weeks):
        """Sets the unpaid_weeks of this EmployeeOpeningBalance.

        The number of unpaid weeks.  # noqa: E501

        :param unpaid_weeks: The unpaid_weeks of this EmployeeOpeningBalance.  # noqa: E501
        :type: int
        """

        self._unpaid_weeks = unpaid_weeks

    @property
    def gross_earnings(self):
        """Gets the gross_earnings of this EmployeeOpeningBalance.  # noqa: E501

        The gross earnings during the period.  # noqa: E501

        :return: The gross_earnings of this EmployeeOpeningBalance.  # noqa: E501
        :rtype: float
        """
        return self._gross_earnings

    @gross_earnings.setter
    def gross_earnings(self, gross_earnings):
        """Sets the gross_earnings of this EmployeeOpeningBalance.

        The gross earnings during the period.  # noqa: E501

        :param gross_earnings: The gross_earnings of this EmployeeOpeningBalance.  # noqa: E501
        :type: float
        """

        self._gross_earnings = gross_earnings



# coding: utf-8

"""
    Xero Files API

    These endpoints are specific to Xero Files API  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class FileObject(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "name": "str",
        "mime_type": "str",
        "size": "int",
        "created_date_utc": "str",
        "updated_date_utc": "str",
        "user": "User",
        "id": "str",
        "folder_id": "str",
    }

    attribute_map = {
        "name": "Name",
        "mime_type": "MimeType",
        "size": "Size",
        "created_date_utc": "CreatedDateUtc",
        "updated_date_utc": "UpdatedDateUtc",
        "user": "User",
        "id": "Id",
        "folder_id": "FolderId",
    }

    def __init__(
        self,
        name=None,
        mime_type=None,
        size=None,
        created_date_utc=None,
        updated_date_utc=None,
        user=None,
        id=None,
        folder_id=None,
    ):  # noqa: E501
        """FileObject - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._mime_type = None
        self._size = None
        self._created_date_utc = None
        self._updated_date_utc = None
        self._user = None
        self._id = None
        self._folder_id = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if mime_type is not None:
            self.mime_type = mime_type
        if size is not None:
            self.size = size
        if created_date_utc is not None:
            self.created_date_utc = created_date_utc
        if updated_date_utc is not None:
            self.updated_date_utc = updated_date_utc
        if user is not None:
            self.user = user
        if id is not None:
            self.id = id
        if folder_id is not None:
            self.folder_id = folder_id

    @property
    def name(self):
        """Gets the name of this FileObject.  # noqa: E501

        File Name  # noqa: E501

        :return: The name of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this FileObject.

        File Name  # noqa: E501

        :param name: The name of this FileObject.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def mime_type(self):
        """Gets the mime_type of this FileObject.  # noqa: E501

        MimeType of the file (image/png, image/jpeg, application/pdf, etc..)  # noqa: E501

        :return: The mime_type of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._mime_type

    @mime_type.setter
    def mime_type(self, mime_type):
        """Sets the mime_type of this FileObject.

        MimeType of the file (image/png, image/jpeg, application/pdf, etc..)  # noqa: E501

        :param mime_type: The mime_type of this FileObject.  # noqa: E501
        :type: str
        """

        self._mime_type = mime_type

    @property
    def size(self):
        """Gets the size of this FileObject.  # noqa: E501

        Numeric value in bytes  # noqa: E501

        :return: The size of this FileObject.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this FileObject.

        Numeric value in bytes  # noqa: E501

        :param size: The size of this FileObject.  # noqa: E501
        :type: int
        """

        self._size = size

    @property
    def created_date_utc(self):
        """Gets the created_date_utc of this FileObject.  # noqa: E501

        Created date in UTC  # noqa: E501

        :return: The created_date_utc of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._created_date_utc

    @created_date_utc.setter
    def created_date_utc(self, created_date_utc):
        """Sets the created_date_utc of this FileObject.

        Created date in UTC  # noqa: E501

        :param created_date_utc: The created_date_utc of this FileObject.  # noqa: E501
        :type: str
        """

        self._created_date_utc = created_date_utc

    @property
    def updated_date_utc(self):
        """Gets the updated_date_utc of this FileObject.  # noqa: E501

        Updated date in UTC  # noqa: E501

        :return: The updated_date_utc of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._updated_date_utc

    @updated_date_utc.setter
    def updated_date_utc(self, updated_date_utc):
        """Sets the updated_date_utc of this FileObject.

        Updated date in UTC  # noqa: E501

        :param updated_date_utc: The updated_date_utc of this FileObject.  # noqa: E501
        :type: str
        """

        self._updated_date_utc = updated_date_utc

    @property
    def user(self):
        """Gets the user of this FileObject.  # noqa: E501


        :return: The user of this FileObject.  # noqa: E501
        :rtype: User
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this FileObject.


        :param user: The user of this FileObject.  # noqa: E501
        :type: User
        """

        self._user = user

    @property
    def id(self):
        """Gets the id of this FileObject.  # noqa: E501

        File object's UUID  # noqa: E501

        :return: The id of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this FileObject.

        File object's UUID  # noqa: E501

        :param id: The id of this FileObject.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def folder_id(self):
        """Gets the folder_id of this FileObject.  # noqa: E501

        Folder relation object's UUID  # noqa: E501

        :return: The folder_id of this FileObject.  # noqa: E501
        :rtype: str
        """
        return self._folder_id

    @folder_id.setter
    def folder_id(self, folder_id):
        """Sets the folder_id of this FileObject.

        Folder relation object's UUID  # noqa: E501

        :param folder_id: The folder_id of this FileObject.  # noqa: E501
        :type: str
        """

        self._folder_id = folder_id



# coding: utf-8

"""
    Xero Assets API

    The Assets API exposes fixed asset related functions of the Xero Accounting application and can be used for a variety of purposes such as creating assets, retrieving asset valuations etc.  # noqa: E501

    Contact: api@xero.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401

from xero_python.models import BaseModel


class FieldValidationErrorsElement(BaseModel):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "field_name": "str",
        "value_provided": "str",
        "localised_message": "str",
        "type": "str",
        "title": "str",
        "detail": "str",
    }

    attribute_map = {
        "field_name": "fieldName",
        "value_provided": "valueProvided",
        "localised_message": "localisedMessage",
        "type": "type",
        "title": "title",
        "detail": "detail",
    }

    def __init__(
        self,
        field_name=None,
        value_provided=None,
        localised_message=None,
        type=None,
        title=None,
        detail=None,
    ):  # noqa: E501
        """FieldValidationErrorsElement - a model defined in OpenAPI"""  # noqa: E501

        self._field_name = None
        self._value_provided = None
        self._localised_message = None
        self._type = None
        self._title = None
        self._detail = None
        self.discriminator = None

        if field_name is not None:
            self.field_name = field_name
        if value_provided is not None:
            self.value_provided = value_provided
        if localised_message is not None:
            self.localised_message = localised_message
        if type is not None:
            self.type = type
        if title is not None:
            self.title = title
        if detail is not None:
            self.detail = detail

    @property
    def field_name(self):
        """Gets the field_name of this FieldValidationErrorsElement.  # noqa: E501

        The field name of the erroneous field  # noqa: E501

        :return: The field_name of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._field_name

    @field_name.setter
    def field_name(self, field_name):
        """Sets the field_name of this FieldValidationErrorsElement.

        The field name of the erroneous field  # noqa: E501

        :param field_name: The field_name of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._field_name = field_name

    @property
    def value_provided(self):
        """Gets the value_provided of this FieldValidationErrorsElement.  # noqa: E501

        The provided value  # noqa: E501

        :return: The value_provided of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._value_provided

    @value_provided.setter
    def value_provided(self, value_provided):
        """Sets the value_provided of this FieldValidationErrorsElement.

        The provided value  # noqa: E501

        :param value_provided: The value_provided of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._value_provided = value_provided

    @property
    def localised_message(self):
        """Gets the localised_message of this FieldValidationErrorsElement.  # noqa: E501

        Explanation of the field validation error  # noqa: E501

        :return: The localised_message of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._localised_message

    @localised_message.setter
    def localised_message(self, localised_message):
        """Sets the localised_message of this FieldValidationErrorsElement.

        Explanation of the field validation error  # noqa: E501

        :param localised_message: The localised_message of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._localised_message = localised_message

    @property
    def type(self):
        """Gets the type of this FieldValidationErrorsElement.  # noqa: E501

        Internal type of the field validation error message  # noqa: E501

        :return: The type of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this FieldValidationErrorsElement.

        Internal type of the field validation error message  # noqa: E501

        :param type: The type of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def title(self):
        """Gets the title of this FieldValidationErrorsElement.  # noqa: E501

        Title of the field validation error  # noqa: E501

        :return: The title of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this FieldValidationErrorsElement.

        Title of the field validation error  # noqa: E501

        :param title: The title of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def detail(self):
        """Gets the detail of this FieldValidationErrorsElement.  # noqa: E501

        Detail of the field validation error  # noqa: E501

        :return: The detail of this FieldValidationErrorsElement.  # noqa: E501
        :rtype: str
        """
        return self._detail

    @detail.setter
    def detail(self, detail):
        """Sets the detail of this FieldValidationErrorsElement.

        Detail of the field validation error  # noqa: E501

        :param detail: The detail of this FieldValidationErrorsElement.  # noqa: E501
        :type: str
        """

        self._detail = detail



# -*- coding: utf-8 -*-
import datetime
import re
from decimal import Decimal
from enum import Enum

from dateutil import tz
from dateutil.parser.isoparser import isoparse, DEFAULT_ISOPARSER

from xero_python.single_dispatch_str import single_dispatch_str

LIST_DATA_TYPE = re.compile(r"list\[(.*)\]")

# taken from https://github.com/django/django/blob/master/django/utils/dateparse.py
# at 4bbe8261c402694a1da3efcaafe3332f9c57af15
DATE_RE = re.compile(r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$")
DATETIME_RE = re.compile(
    r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})"
    r"[T ](?P<hour>\d{1,2}):(?P<minute>\d{1,2})"
    r"(?::(?P<second>\d{1,2})(?:[\.,](?P<microsecond>\d{1,6})\d{0,6})?)?"
    r"(?P<tzinfo>Z|[+-]\d{2}(?::?\d{2})?)?$"
)

MS_DATETIME_RE = re.compile(r"/Date\((?P<timestamp>-?\d+)(?P<tzinfo>[+-]\d{2,4})?\)/$")


def deserialize_routing(data_type, data, model_finder):
    """Custom logic to find matching deserialize implementation and
       returns it's unique registration string key
    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: str key to find proper deserialize implementation
    """
    if LIST_DATA_TYPE.match(data_type):
        return "list"
    return data_type


@single_dispatch_str(key=deserialize_routing)
def deserialize(data_type, data, model_finder):
    """Deserializes data into an object.
       Default implementation search model class for given data type

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized object.

    """
    try:
        model = model_finder.find_model(data_type)
    except AttributeError:
        raise ValueError("Can't deserialize data_type={!r}".format(data_type))
    else:
        return deserialize_model(model, data, model_finder)


@deserialize.register("list")
def deserialize_list(data_type, data, model_finder):
    """Deserializes data into a list of objects.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized list

    """
    try:
        sub_data_type = LIST_DATA_TYPE.match(data_type).group(1)
    except AttributeError:
        raise ValueError("Can't deserialize data_type={!r}".format(data_type))
    else:
        return [deserialize(sub_data_type, sub_data, model_finder) for sub_data in data]


@deserialize.register("int")
def deserialize_int(data_type, data, model_finder):
    """Deserializes data into an integer number.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized int

    """
    return int(data)


@deserialize.register("float")
def deserialize_decimal(data_type, data, model_finder):
    """Deserializes data into a Decimal number.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized Decimal

    """
    if data is not None:
        return data if isinstance(data, Decimal) else Decimal(data)

    return data


@deserialize.register("str")
def deserialize_str(data_type, data, model_finder):
    """Deserializes data into a string

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized str

    """
    return str(data)


@deserialize.register("bool")
def deserialize_bool(data_type, data, model_finder):
    """Deserializes data into a boolean object.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized bool

    """
    # allowed json payload is true or false - parsed to bool via json.loads
    if data is not None:
        if not isinstance(data, bool):
            raise ValueError("Json parsed bool value expected. got {!r}".format(data))

    return bool(data)


@deserialize.register("date")
def deserialize_date(data_type, data, model_finder):
    """Deserializes data into a python date
       Expected input value in format YYYY-MM-DD as per
       https://tools.ietf.org/html/rfc3339#section-5.6 standard

       Raise ValueError if the input is well formatted but not a valid date.
       Return None if the input isn't well formatted.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized datetime.date

    """
    # based on https://github.com/django/django/blob/master/django/utils/dateparse.py
    # at 4bbe8261c402694a1da3efcaafe3332f9c57af15
    match = DATE_RE.match(str(data))
    match2 = DATETIME_RE.match(str(data))

    if data is not None:
        if match:
            kw = {k: int(v) for k, v in match.groupdict().items()}
            return datetime.date(**kw)
        elif match2:
            dt = isoparse(data)
            return dt
        else:
            raise ValueError("Invalid date value {!r}".format(data))


@deserialize.register("date[ms-format]")
def deserialize_date_ms(data_type, data, model_finder):
    """Deserializes data into a python date
       Expected input value in format /Date(timestamp_in_milliseconds)/

       Raise ValueError if the input is well formatted but not a valid date.
       Return None if the input isn't well formatted.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized datetime.date

    """
    ms_datetime = deserialize_datetime_ms(data_type, data, model_finder)
    # shift datetime to utc timezone to make sure utc date used
    return ms_datetime.astimezone(tz=tz.UTC).date()


@deserialize.register("datetime")
def deserialize_datetime(data_type, data, model_finder):
    """Deserializes data into a python datetime

       Expected input value in format YYYY-MM-DDTHH:mm:ss[Z] as per
       https://tools.ietf.org/html/rfc3339#section-5.6 standard

       Raise ValueError if the input is well formatted but not a valid datetime.
       Return None if the input isn't well formatted.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized datetime.datetime

    """
    try:
        dt = isoparse(data)
    except (ValueError, TypeError):
        raise ValueError("Invalid datetime value {!r}".format(data))

    if not dt.tzinfo:
        # timezone naive datetime from Xero API response always in UTC
        dt = dt.replace(tzinfo=tz.UTC)

    return dt


@deserialize.register("datetime[ms-format]")
def deserialize_datetime_ms(data_type, data, model_finder):
    """Deserializes data into a python datetime

       Expected input value in format /Date(timestamp_in_milliseconds)/

       Raise ValueError if the input is well formatted but not a valid datetime.
       Return None if the input isn't well formatted.

    :param data_type: class literal for deserialized object, or string of class name
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for data_type class literal

    :return: deserialized datetime.datetime

    """
    match = MS_DATETIME_RE.match(str(data))
    if match:
        tz_str = match.groupdict()["tzinfo"]
        if tz_str:
            tz_info = DEFAULT_ISOPARSER.parse_tzstr(tz_str)
        else:
            # timezone naive datetime from Xero API response always in UTC
            tz_info = tz.UTC

        timestamp_ms = int(match.groupdict()["timestamp"])
        timestamp_s = timestamp_ms / 1000
        return datetime.datetime.fromtimestamp(timestamp_s, tz=tz_info)
    else:
        raise ValueError("Invalid datetime value {!r}".format(data))


def deserialize_model(model, data, model_finder):
    """Deserializes data into model instance.

    :param model: class object
    :param data: data to be parsed
    :param model_finder: ModelFinder instance to find class for class literal

    :return: deserialized model instance.

    """
    if data is None:
        return model("")

    if issubclass(model, Enum):
        return model(data)

    if not isinstance(data, (list, dict)):
        raise ValueError("list or dict value expected. got {!r}".format(data))

    kwargs = {}
    for attr, attr_type in model.openapi_types.items():
        attr_key = model.attribute_map[attr]
        if attr_key in data:
            value = data[attr_key]
            kwargs[attr] = deserialize(attr_type, value, model_finder)

    instance = model(**kwargs)
    return instance
